// Code generated by go tool dist; DO NOT EDIT.
// This is a bootstrap copy of /tmp/build/80754af9/go_1537369800302/work/go/src/cmd/compile/internal/gc/norace.go

//line /tmp/build/80754af9/go_1537369800302/work/go/src/cmd/compile/internal/gc/norace.go:1
// Copyright 2017 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// +build !race

package gc

const raceEnabled = false
