// Code generated by go tool dist; DO NOT EDIT.
// This is a bootstrap copy of /tmp/build/80754af9/go_1537369800302/work/go/src/cmd/compile/internal/ssa/stackframe.go

//line /tmp/build/80754af9/go_1537369800302/work/go/src/cmd/compile/internal/ssa/stackframe.go:1
// Copyright 2016 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package ssa

// stackframe calls back into the frontend to assign frame offsets.
func stackframe(f *Func) {
	f.fe.AllocFrame(f)
}
