// Code generated by go tool dist; DO NOT EDIT.
// This is a bootstrap copy of /tmp/build/80754af9/go_1537369800302/work/go/src/cmd/internal/objabi/zbootstrap.go

//line /tmp/build/80754af9/go_1537369800302/work/go/src/cmd/internal/objabi/zbootstrap.go:1
// Code generated by go tool dist; DO NOT EDIT.

package objabi

import "runtime"

const defaultGO386 = `sse2`
const defaultGOARM = `5`
const defaultGOMIPS = `hardfloat`
const defaultGOOS = runtime.GOOS
const defaultGOARCH = runtime.GOARCH
const defaultGO_EXTLINK_ENABLED = ``
const version = `go1.10.3`
const stackGuardMultiplier = 1
const goexperiment = ``
